@@ -234,7 +234,8 @@ void parentname2(char* result, int result_size, char* path)
 #include <migemo.h>
 
 migemo* gMigemo;
-static regex_t* gReg;
+
+hash_obj* gMigemoRegsCache;
 
 void migemo_init()
 {
@@ -310,14 +311,13 @@ void migemo_init()
             exit(1);
         }
     }
+    gMigemoRegsCache = HASH_NEW(100);
 }
 
 void migemo_final()
 {
     onig_end();
     migemo_close(gMigemo);
-    if(gReg) onig_free(gReg);
-    gReg = NULL;
 }
 
 /// 先頭から同じ文字列があるかどうか調べる。
@@ -338,13 +338,19 @@ int str_headsame(char* str1, char* str2)
 }
 
 
-char* readline_file_completion_migemo_match(char* text, char* file)
+char* readline_file_completion_migemo_match(char* text, char* file, regex_t* reg)
 {
     int same_point = str_headsame(text, file);
 
-    OnigUChar * p;
-    if(same_point == 0) {
-        p = migemo_query(gMigemo, text);
+    int r2;
+    if(reg && same_point == 0) {
+        OnigRegion* region = onig_region_new();
+        
+        OnigUChar* file2 = (OnigUChar*)file;
+
+        r2 = onig_search(reg, file2, file2 + strlen((char*)file2), file2, file2 + strlen((char*)file2), region, ONIG_OPTION_NONE);
+
+        onig_region_free(region, 1);
     }
     /// text側がfileの部分なら判定するまでも無くマッチ ///
     else if(same_point == strlen(text)) {
@@ -353,67 +359,77 @@ char* readline_file_completion_migemo_match(char* text, char* file)
         return strdup(tmp);
     }
     else {
-        p = migemo_query(gMigemo, text+same_point);
-    }
+        OnigUChar * p = migemo_query(gMigemo, text+same_point);
 
-
-    if(p == NULL) {
-        return NULL;
-    }
-
-    static regex_t* reg;
-
-    int r;
-    OnigErrorInfo err_info;
-
-    char* p2 = MALLOC(strlen(p)*2);
-
-    char* _p = p;
-    char* _p2 = p2;
-
-    while(*_p) {
-        if(*_p == '+') {
-            *_p2++ = '\\';
-            *_p2++ = *_p++;
+        if(p == NULL) {
+           return NULL;
         }
-        else {
-            *_p2++ = *_p++;
+
+        int r;
+        OnigErrorInfo err_info;
+
+        char* p2 = MALLOC(strlen(p)*2);
+
+        char* _p = p;
+        char* _p2 = p2;
+
+        while(*_p) {
+            if(*_p == '+') {
+                *_p2++ = '\\';
+                *_p2++ = *_p++;
+            }
+            else {
+                *_p2++ = *_p++;
+            }
         }
-    }
-    *_p2 = 0;
+        *_p2 = 0;
 
-    if(gKanjiCode == kUtf8) {
-        r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
-    }
-    else if(gKanjiCode == kEucjp) {
-        r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
-    }
-    else if(gKanjiCode == kSjis) {
-        r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
-    }
+        regex_t* reg;
+        if(strlen(p2) == 2) {
+            reg = hash_item(gMigemoRegsCache, p2);
 
-    migemo_release(gMigemo, (unsigned char*) p);
+            if(reg == NULL) {
+                if(gKanjiCode == kUtf8) {
+                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
+                }
+                else if(gKanjiCode == kEucjp) {
+                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
+                }
+                else if(gKanjiCode == kSjis) {
+                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
+                }
 
-    if(r != ONIG_NORMAL) {
-        return NULL;
+                hash_put(gMigemoRegsCache, p2, reg);
+            }
+        }
+        else { 
+            if(gKanjiCode == kUtf8) {
+                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
+            }
+            else if(gKanjiCode == kEucjp) {
+                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
+            }
+            else if(gKanjiCode == kSjis) {
+                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
+            }
+        }
+
+        migemo_release(gMigemo, (unsigned char*) p);
+
+        if(r != ONIG_NORMAL) {
+            return NULL;
+        }
+        OnigRegion* region = onig_region_new();
+        
+        OnigUChar* file2 = (OnigUChar*)file + same_point;
+
+        r2 = onig_search(reg, file2, file2 + strlen((char*)file2), file2, file2 + strlen((char*)file2), region, ONIG_OPTION_NONE);
+
+        onig_region_free(region, 1);
+
+        if(strlen(p2) > 1) { onig_free(reg); }
     }
 
-    OnigRegion* region = onig_region_new();
-    
-    OnigUChar* file2;
-    if(same_point == 0) {
-        file2 = (OnigUChar*)file;
-    }
-    else {
-        file2 = (OnigUChar*)file + same_point;
-    }
-
-    int r2 = onig_search(reg, file2, file2 + strlen((char*)file2), file2, file2 + strlen((char*)file2), region, ONIG_OPTION_NONE);
-
-    onig_region_free(region, 1);
-
-    onig_free(reg);
-
     if(r2 == 0) {
         char tmp[PATH_MAX];
         xstrncpy(tmp, file, PATH_MAX);
@@ -442,12 +458,76 @@ char* readline_file_completion_migemo(const char* text, int rstat)
     *_p2 = 0;
 
     static int index, wordlen;
+    static regex_t* reg = NULL;
 
     /// 初期化 ///
     if(rstat == 0) {
         wordlen = strlen(text2);
         index = 0;
 
+        /// migemoのクェリーと正規表現コンパイル結果を得る
+        if(reg) { if(hash_key(gMigemoRegsCache, reg) == NULL) { onig_free(reg);} }
+
+        OnigUChar* p = migemo_query(gMigemo, text2);
+
+        if(p == NULL) {
+           return NULL;
+        }
+
+        int r;
+        OnigErrorInfo err_info;
+
+        char* p2 = MALLOC(strlen(p)*2);
+        char* _p = p;
+        char* _p2 = p2;
+
+        while(*_p) {
+            if(*_p == '+') {
+                *_p2++ = '\\';
+                *_p2++ = *_p++;
+            }
+            else {
+                *_p2++ = *_p++;
+            }
+        }
+        *_p2 = 0;
+
+
+        if(strlen(p2) == 2) {
+            reg = hash_item(gMigemoRegsCache, p2);
+
+            if(reg == NULL) {
+                if(gKanjiCode == kUtf8) {
+                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
+                }
+                else if(gKanjiCode == kEucjp) {
+                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
+                }
+                else if(gKanjiCode == kSjis) {
+                    r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
+                }
+
+                hash_put(gMigemoRegsCache, p2, reg);
+            }
+        }
+        else { 
+            if(gKanjiCode == kUtf8) {
+                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
+            }
+            else if(gKanjiCode == kEucjp) {
+                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_EUC_JP, ONIG_SYNTAX_DEFAULT, &err_info);
+            }
+            else if(gKanjiCode == kSjis) {
+                r = onig_new(&reg, p2, p2 + strlen((char*)p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_SJIS, ONIG_SYNTAX_DEFAULT, &err_info);
+            }
+        }
+
+        migemo_release(gMigemo, (unsigned char*) p);
+
+        if(r != ONIG_NORMAL) {
+            reg = NULL;
+        }
+
         // ディレクトリのパートを得る
         string_put(gEditingDir, "");
         read_one_argument(rl_line_buffer, gEditingDir);
@@ -615,12 +695,12 @@ char* readline_file_completion_migemo(const char* text, int rstat)
             char path[PATH_MAX];
             xstrncpy(path, file, PATH_MAX);
 
-            char* ret = strdup(path);
-
+            char path2[PATH_MAX];
             struct stat stat_;
-            xstrncpy(path, string_c_str(gEditingDir), PATH_MAX);
-            xstrncat(path, file, PATH_MAX);
-            if(stat(path, &stat_) == 0) {
+
+            xstrncpy(path2, string_c_str(gEditingDir), PATH_MAX);
+            xstrncat(path2, file, PATH_MAX);
+            if(stat(path2, &stat_) == 0) {
                 if(S_ISDIR(stat_.st_mode)) {
                     rl_completion_append_character= 0;
                 }
@@ -632,12 +712,16 @@ char* readline_file_completion_migemo(const char* text, int rstat)
                 rl_completion_append_character= ' ';
             }
 
+            string_obj* tmp2 = STRING_NEW("");
+            saphire_get_quoted_fname(path, tmp2);
+            char* ret = strdup(string_c_str(tmp2));
+
             return ret;
         }
 
         /// アスキー文字同士の補完 ///
         else if(is_all_ascii((char*)text2) && is_all_ascii(file)) {
-            if(!strncmp(text2, file, wordlen)) {
+            if(strncmp(text2, file, wordlen) == 0) {
                 char path[PATH_MAX];
                 xstrncpy(path, file, PATH_MAX);
 
@@ -665,7 +749,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         }
         /// migemo 補完 ///
         else {
-            char* ret = readline_file_completion_migemo_match((char*)text2 , file);
+            char* ret = readline_file_completion_migemo_match((char*)text2 , file, reg);
             if(ret) {
                 struct stat stat_;
                 char path[PATH_MAX];
@@ -698,7 +782,6 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         return strdup("");
     }
 
-
     return NULL;
 }
 #endif
@@ -1243,7 +1326,6 @@ char* readline_file_completion(const char* text, int rstat)
         }
 
         /// アスキー文字同士の補完 ///
-//        else if(is_all_ascii((char*)text2) && is_all_ascii(file)) {
         else {
             if(!strncmp(text2, file, wordlen)) {
                 char path[PATH_MAX];
