@@ -385,22 +385,6 @@ string_obj* sGlobal_new(char* str);
 vector_obj* sAry_new(int first_size);
 hash_obj* sHash_new(int size);
 
-/// å
-void memchecker_init();
-enum eMemCheckerKind { kMCNothing, kMCRef, kMCArray, kMCObject, kMCHash, kMCVar, kMCGlobal, kMCFunction, kMCClass };
-
-void* memchecker_GC_malloc(int size, enum eMemCheckerKind obj_kind);
-    // åȤäGCΥΰ
-int memchecker_is_enable_mem(void* obj);
-    // ֥Ȥμ֤ 0:̵ 1:string_obj*
-
-typedef struct {
-    void* mMem;
-    enum eMemCheckerKind mKind;
-} sRef;
-
-sRef* sRef_new(void* mem, enum eMemCheckerKind kind);
-
 ////////////////////////////////////////////////////////////
 /// from saphire_commands.c
 ////////////////////////////////////////////////////////////
