@@ -68,7 +68,7 @@ char* gStatmentKindStrs[kCommand+1] = {
     "/",
     "mod",
     "pow",
-    "sort",
+    "ssort",
     "range",
     "printf",
     "pomch",
@@ -2540,7 +2540,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "3.4.7", 1);
+    setenv("SAPHIRE_VERSION", "3.4.8", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
