@@ -581,6 +581,16 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
         }
     }
 
+    if(squote) {
+        err_msg("require to close '", sname, *sline);
+        return FALSE;
+    }
+
+    if(dquote) {
+        err_msg("require to close \"", sname, *sline);
+        return FALSE;
+    }
+
     return TRUE;
 }
 
