@@ -21,18 +21,9 @@ typedef struct {
 // ֐`
 //////////////////////////////////////////////////////////////////////
 // 
-#if defined(MDEBUG)
-
-vector_obj* vector_new_debug(int first_size, const char* fname, int line, const char* func_name);
-#define VECTOR_NEW(size) vector_new_debug(size, __FILE__, __LINE__, __FUNCTION__)
-
-#else
-
 vector_obj* vector_new(int first_size);
 #define VECTOR_NEW(size) vector_new(size)
 
-#endif
-
 void vector_delete(vector_obj* self);                           // 
 
 vector_obj* vector_clone(vector_obj* v);                        // N[
