@@ -837,16 +837,22 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             tmp2[tmp - arg0] = 0;
 
             if(strlen(tmp2) > 1 && tmp2[0] == '$') {
-                sRef* ref2 = NULL;
+                sRef* ref2;
                 hash_obj* top_stack
                      = vector_item(gStackFrameRefs, vector_size(gStackFrameRefs)-1);
-                if(hash_item(top_stack, tmp2 + 1)) {
-                    ref2 = hash_item(top_stack, tmp2 + 1);
-                }
-                else if(object && hash_item(object->mRefs, tmp2 + 1)) {
+                ref2 = hash_item(top_stack, tmp2 + 1);
+                if(ref2 == NULL && object) {
                     ref2 = hash_item(object->mRefs, tmp2 + 1);
+                    if(ref2 == NULL) {
+                        sObject* parent = object->mParent;
+                        while(parent) {
+                            ref2 = hash_item(parent->mRefs, tmp2 + 1);
+                            if(ref2) break;
+                            parent = parent->mParent;
+                        }
+                    }
                 }
-                else {
+                if(ref2 == NULL) {
                     ref2 = hash_item(gRefs, tmp2 + 1);
                 }
 
@@ -928,16 +934,22 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
         }
         else {
             if(strlen(arg0) > 1 && arg0[0] == '$') {
-                sRef* ref2 = NULL;
+                sRef* ref2;
                 hash_obj* top_stack
                      = vector_item(gStackFrameRefs, vector_size(gStackFrameRefs)-1);
-                if(hash_item(top_stack, arg0 + 1)) {
-                    ref2 = hash_item(top_stack, arg0 + 1);
-                }
-                else if(object && hash_item(object->mRefs, arg0 + 1)) {
+                ref2 = hash_item(top_stack, arg0 + 1);
+                if(ref2 == NULL && object) {
                     ref2 = hash_item(object->mRefs, arg0 + 1);
+                    if(ref2 == NULL) {
+                        sObject* parent = object->mParent;
+                        while(parent) {
+                            ref2 = hash_item(parent->mRefs, arg0 + 1);
+                            if(ref2) break;
+                            parent = parent->mParent;
+                        }
+                    }
                 }
-                else {
+                if(ref2 == NULL) {
                     ref2 = hash_item(gRefs, arg0 + 1);
                 }
 
@@ -1067,6 +1079,18 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 vector_add(gStackFrameObjects, HASH_NEW(30));
                 vector_add(gStackFrameRefs, HASH_NEW(30));
 
+                /// 引数を渡す ///
+                hash_obj* fun_argv_before = hash_item(object2->mArrays, "ARGV");
+
+                int k;
+                vector_obj* argv2 = VECTOR_NEW(30);
+                for(k=2; k<vector_size(argv); k++) {
+                    char* value = string_c_str(vector_item(argv, k));
+                    vector_add(argv2, STRING_NEW(value));
+                }
+                
+                hash_put(object2->mArrays, "ARGV", argv2);
+
                 *rcode = run(klass->statments, title, nextout
                                 , nextin, nexterr
                                 , return_, TRUE, break_, FALSE, parent_blocks, FALSE, envs, object2, NULL, klass);
@@ -1083,6 +1107,17 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 /// スタックフレームの記録を消す
                 string_obj* sname_strace = vector_pop_back(gStackTraceFName);
                 int line_strace = (int)vector_pop_back(gStackTraceLineNum);
+                /// 引数を戻す
+                hash_put(object2->mArrays, "ARGV", fun_argv_before);
+
+                if(*rcode < 0) {
+                    string_push_back(gErrMsg, string_c_str(sname_strace));
+                    char tmp[1024];
+                    snprintf(tmp, 1024, ":%d\n", line_strace);
+                    string_push_back(gErrMsg, tmp);
+
+                    return FALSE;
+                }
 
                 /// 終了コード ///
                 if(statment->mRCodeReverse) {
@@ -1095,6 +1130,60 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 char buf2[256];
                 snprintf(buf2, 256, "%d", *rcode);
                 saphire_set_global_var("RCODE", buf2);
+            }
+            else {
+                err_msg("reuire object name", sname, sline);
+                return FALSE;
+            }
+        }
+        /// classメソッド ///
+        else if(method_class) {
+            if(vector_size(blocks) == 1) {
+                /// 引数を渡す ///
+                hash_obj* fun_argv_before = hash_item(object2->mArrays, "ARGV");
+
+                int k;
+                vector_obj* argv2 = VECTOR_NEW(30);
+                for(k=1; k<vector_size(argv); k++) {
+                    char* value = string_c_str(vector_item(argv, k));
+                    vector_add(argv2, STRING_NEW(value));
+                }
+                
+                hash_put(object2->mArrays, "ARGV", argv2);
+
+                /// スタックフレームの記録を付ける
+                vector_add(gStackTraceFName, STRING_NEW(sname));
+                vector_add(gStackTraceLineNum, (void*)sline);
+
+                // スタックを掘る
+                vector_add(gStackFrame, HASH_NEW(30));
+                vector_add(gStackFrameHashs, HASH_NEW(30));
+                vector_add(gStackFrameArrays, HASH_NEW(30));
+                vector_add(gStackFrameObjects, HASH_NEW(30));
+                vector_add(gStackFrameRefs, HASH_NEW(30));
+
+                sBlock* block = vector_item(blocks, 0);
+                sStatments* statments = block->mStatments;
+                
+                *rcode = run(statments, title, nextout
+                                , nextin, nexterr
+                                , return_, TRUE, break_, FALSE, parent_blocks, FALSE, envs, object2, NULL, NULL);
+                *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
+                *break_ = FALSE;
+
+                // スタックを埋める
+                vector_pop_back(gStackFrame);
+                vector_pop_back(gStackFrameArrays);
+                vector_pop_back(gStackFrameObjects);
+                vector_pop_back(gStackFrameHashs);
+                vector_pop_back(gStackFrameRefs);
+
+                /// スタックフレームの記録を消す
+                string_obj* sname_strace = vector_pop_back(gStackTraceFName);
+                int line_strace = (int)vector_pop_back(gStackTraceLineNum);
+
+                /// 引数を戻す
+                hash_put(object2->mArrays, "ARGV", fun_argv_before);
 
                 if(*rcode < 0) {
                     string_push_back(gErrMsg, string_c_str(sname_strace));
@@ -1104,15 +1193,20 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
                     return FALSE;
                 }
+
+                /// 終了コード ///
+                if(statment->mRCodeReverse) {
+                    if(*rcode == 0) 
+                        *rcode = 1;
+                    else
+                        *rcode = 0;
+                }
+
+                char buf2[256];
+                snprintf(buf2, 256, "%d", *rcode);
+                saphire_set_global_var("RCODE", buf2);
             }
-            else {
-                err_msg("reuire object name", sname, sline);
-                return FALSE;
-            }
-        }
-        /// 継承 ///
-        else if(method_class) {
-            if(vector_size(argv) >= 2) {
+            else if(vector_size(argv) >= 2) {
                 char* klassname = string_c_str(vector_item(argv, 1));
 
                 /// スタックフレームの記録を付ける
@@ -1126,6 +1220,18 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 vector_add(gStackFrameObjects, HASH_NEW(30));
                 vector_add(gStackFrameRefs, HASH_NEW(30));
 
+                /// 引数を渡す ///
+                hash_obj* fun_argv_before = hash_item(object2->mArrays, "ARGV");
+
+                int k;
+                vector_obj* argv2 = VECTOR_NEW(30);
+                for(k=2; k<vector_size(argv); k++) {
+                    char* value = string_c_str(vector_item(argv, k));
+                    vector_add(argv2, STRING_NEW(value));
+                }
+                
+                hash_put(object2->mArrays, "ARGV", argv2);
+
                 if(object) {
                     klass = hash_item(object->mClasses, klassname);
                     if(klass == NULL) {
@@ -1157,6 +1263,9 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
                 *break_ = FALSE;
 
+                /// 引数を戻す
+                hash_put(object2->mArrays, "ARGV", fun_argv_before);
+
                 // スタックを埋める
                 vector_pop_back(gStackFrame);
                 vector_pop_back(gStackFrameArrays);
@@ -1167,6 +1276,14 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 /// スタックフレームの記録を消す
                 string_obj* sname_strace = vector_pop_back(gStackTraceFName);
                 int line_strace = (int)vector_pop_back(gStackTraceLineNum);
+                if(*rcode < 0) {
+                    string_push_back(gErrMsg, string_c_str(sname_strace));
+                    char tmp[1024];
+                    snprintf(tmp, 1024, ":%d\n", line_strace);
+                    string_push_back(gErrMsg, tmp);
+
+                    return FALSE;
+                }
 
                 /// 終了コード ///
                 if(statment->mRCodeReverse) {
@@ -1179,68 +1296,6 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 char buf2[256];
                 snprintf(buf2, 256, "%d", *rcode);
                 saphire_set_global_var("RCODE", buf2);
-
-                if(*rcode < 0) {
-                    string_push_back(gErrMsg, string_c_str(sname_strace));
-                    char tmp[1024];
-                    snprintf(tmp, 1024, ":%d\n", line_strace);
-                    string_push_back(gErrMsg, tmp);
-
-                    return FALSE;
-                }
-            }
-            else if(vector_size(blocks) == 1) {
-                /// スタックフレームの記録を付ける
-                vector_add(gStackTraceFName, STRING_NEW(sname));
-                vector_add(gStackTraceLineNum, (void*)sline);
-
-                // スタックを掘る
-                vector_add(gStackFrame, HASH_NEW(30));
-                vector_add(gStackFrameHashs, HASH_NEW(30));
-                vector_add(gStackFrameArrays, HASH_NEW(30));
-                vector_add(gStackFrameObjects, HASH_NEW(30));
-                vector_add(gStackFrameRefs, HASH_NEW(30));
-
-                sBlock* block = vector_item(blocks, 0);
-                sStatments* statments = block->mStatments;
-                
-                *rcode = run(statments, title, nextout
-                                , nextin, nexterr
-                                , return_, TRUE, break_, FALSE, parent_blocks, FALSE, envs, object2, NULL, NULL);
-                *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
-                *break_ = FALSE;
-
-                // スタックを埋める
-                vector_pop_back(gStackFrame);
-                vector_pop_back(gStackFrameArrays);
-                vector_pop_back(gStackFrameObjects);
-                vector_pop_back(gStackFrameHashs);
-                vector_pop_back(gStackFrameRefs);
-
-                /// スタックフレームの記録を消す
-                string_obj* sname_strace = vector_pop_back(gStackTraceFName);
-                int line_strace = (int)vector_pop_back(gStackTraceLineNum);
-
-                /// 終了コード ///
-                if(statment->mRCodeReverse) {
-                    if(*rcode == 0) 
-                        *rcode = 1;
-                    else
-                        *rcode = 0;
-                }
-
-                char buf2[256];
-                snprintf(buf2, 256, "%d", *rcode);
-                saphire_set_global_var("RCODE", buf2);
-
-                if(*rcode < 0) {
-                    string_push_back(gErrMsg, string_c_str(sname_strace));
-                    char tmp[1024];
-                    snprintf(tmp, 1024, ":%d\n", line_strace);
-                    string_push_back(gErrMsg, tmp);
-
-                    return FALSE;
-                }
             }
             else {
                 err_msg("reuire class name or a block", sname, sline);
@@ -1349,6 +1404,15 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 }
             }
 
+            if(*rcode == -1) {
+                string_push_back(gErrMsg, string_c_str(sname_strace));
+                char tmp[1024];
+                snprintf(tmp, 1024, ":%d\n", line_strace);
+                string_push_back(gErrMsg, tmp);
+                
+                return FALSE;
+            }
+
             /// 終了コード ///
             if(statment->mRCodeReverse) {
                 if(*rcode == 0) 
@@ -1360,15 +1424,6 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             char buf2[256];
             snprintf(buf2, 256, "%d", *rcode);
             saphire_set_global_var("RCODE", buf2);
-
-            if(*rcode == -1) {
-                string_push_back(gErrMsg, string_c_str(sname_strace));
-                char tmp[1024];
-                snprintf(tmp, 1024, ":%d\n", line_strace);
-                string_push_back(gErrMsg, tmp);
-                
-                return FALSE;
-            }
         }
 
         /// ユーザーコマンド ///
@@ -1472,6 +1527,15 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 }
             }
 
+            if(*rcode == -1) {
+                string_push_back(gErrMsg, string_c_str(sname_strace));
+                char tmp[1024];
+                snprintf(tmp, 1024, ":%d\n", line_strace);
+                string_push_back(gErrMsg, tmp);
+                
+                return FALSE;
+            }
+
             /// 終了コード ///
             if(statment->mRCodeReverse) {
                 if(*rcode == 0) 
@@ -1483,15 +1547,6 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             char buf2[256];
             snprintf(buf2, 256, "%d", *rcode);
             saphire_set_global_var("RCODE", buf2);
-
-            if(*rcode == -1) {
-                string_push_back(gErrMsg, string_c_str(sname_strace));
-                char tmp[1024];
-                snprintf(tmp, 1024, ":%d\n", line_strace);
-                string_push_back(gErrMsg, tmp);
-                
-                return FALSE;
-            }
         }
 
         /// 外部登録の内部コマンド ///
@@ -1530,6 +1585,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             char buf[256];
             snprintf(buf, 256, "%d", *rcode);
             saphire_set_global_var("RCODE", buf);
+
             if(!r) {
                 return FALSE;
             }
